from typing import List, Optional

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app.db.session import get_db
from app.db import crud
from app.schemas.todo import Todo, TodoCreate, TodoUpdate

router = APIRouter()


@router.get("/", response_model=List[Todo])
def read_todos(
    skip: int = 0, 
    limit: int = 100, 
    db: Session = Depends(get_db)
):
    """
    Retrieve todos.
    """
    todos = crud.get_todos(db, skip=skip, limit=limit)
    return todos


@router.post("/", response_model=Todo, status_code=status.HTTP_201_CREATED)
def create_todo(
    todo: TodoCreate, 
    db: Session = Depends(get_db)
):
    """
    Create a new todo.
    """
    return crud.create_todo(db=db, todo=todo)


@router.get("/{todo_id}", response_model=Todo)
def read_todo(
    todo_id: int, 
    db: Session = Depends(get_db)
):
    """
    Get a specific todo by id.
    """
    db_todo = crud.get_todo(db, todo_id=todo_id)
    if db_todo is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, 
            detail="Todo not found"
        )
    return db_todo


@router.put("/{todo_id}", response_model=Todo)
def update_todo(
    todo_id: int, 
    todo: TodoUpdate, 
    db: Session = Depends(get_db)
):
    """
    Update a todo.
    """
    db_todo = crud.update_todo(db, todo_id=todo_id, todo=todo)
    if db_todo is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, 
            detail="Todo not found"
        )
    return db_todo


@router.delete("/{todo_id}", status_code=status.HTTP_204_NO_CONTENT)
def delete_todo(
    todo_id: int, 
    db: Session = Depends(get_db)
):
    """
    Delete a todo.
    """
    deleted = crud.delete_todo(db, todo_id=todo_id)
    if not deleted:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, 
            detail="Todo not found"
        )
    return None